@@ -1,7 +1,8 @@
 /* GNOME (analog) clock applet
- * (C) 1999 Iigo Serna
+ * (C) 1999 Free Software Foundation
  *
- * Author: Iigo Serna
+ * Author: Iigo Serna <inigo@gaztelan.bi.ehu.es>,
+ *	   with some code by Miguel de Icaza <miguel@kernel.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,15 +40,15 @@
 
 
 struct clock_props_t {
-   gboolean	sec;
-   gushort	red[4], blue[4], green[4];	/* bg, hour, min, sec */
+   gboolean	secneedle;
+   gchar	*bg, *hour, *min, *sec;
 };
 
 struct clock_applet_t{
    GtkWidget		*area;
+   GtkWidget		*clock;
+   GtkWidget		*pixmap;
    GdkPixmap		*xpm;
-   GdkPixmap		*clock_xpm;
-   GdkBitmap		*clock_mask;
    GdkGC		*gc[4];		/* bg, hour, min, sec */
    struct clock_props_t	props;
 };
@@ -73,19 +74,11 @@ void set_colors (GtkWidget *widget);
 void properties_load (char *path)
 {
     gnome_config_push_prefix (path);
-    clk.props.sec	= gnome_config_get_bool_with_default ("another_clock/sec_needle=TRUE", NULL);
-    clk.props.red[0]	= gnome_config_get_int_with_default ("another_clock/background_red=37888", NULL);
-    clk.props.green[0]	= gnome_config_get_int_with_default ("another_clock/background_green=0", NULL);
-    clk.props.blue[0]	= gnome_config_get_int_with_default ("another_clock/background_blue=54016", NULL);
-    clk.props.red[1]	= gnome_config_get_int_with_default ("another_clock/hourneedle_red=65535", NULL);
-    clk.props.green[1]	= gnome_config_get_int_with_default ("another_clock/hourneedle_green=65535", NULL);
-    clk.props.blue[1]	= gnome_config_get_int_with_default ("another_clock/hourneedle_blue=65535", NULL);
-    clk.props.red[2]	= gnome_config_get_int_with_default ("another_clock/minneedle_red=65535", NULL);
-    clk.props.green[2]	= gnome_config_get_int_with_default ("another_clock/minneedle_green=65535", NULL);
-    clk.props.blue[2]	= gnome_config_get_int_with_default ("another_clock/minneedle_blue=65535", NULL);
-    clk.props.red[3]	= gnome_config_get_int_with_default ("another_clock/secneedle_red=65280", NULL);
-    clk.props.green[3]	= gnome_config_get_int_with_default ("another_clock/secneedle_green=25344", NULL);
-    clk.props.blue[3]	= gnome_config_get_int_with_default ("another_clock/secneedle_blue=18176", NULL);
+    clk.props.secneedle	= gnome_config_get_bool_with_default ("another_clock/sec_needle=TRUE", NULL);
+    clk.props.bg 	= gnome_config_get_string_with_default ("another_clock/background=#68228b", NULL);
+    clk.props.hour 	= gnome_config_get_string_with_default ("another_clock/hourneedle=#ffffff", NULL);
+    clk.props.min 	= gnome_config_get_string_with_default ("another_clock/minneedle=#ffffff", NULL);
+    clk.props.sec 	= gnome_config_get_string_with_default ("another_clock/secneedle=#ff4500", NULL);
     gnome_config_pop_prefix();
 }
 
@@ -93,19 +86,11 @@ void properties_load (char *path)
 int properties_save (char *path)
 {
     gnome_config_push_prefix (path);
-    gnome_config_set_bool ("another_clock/sec_needle", clk.props.sec);
-    gnome_config_set_int ("another_clock/background_red", clk.props.red[0]);
-    gnome_config_set_int ("another_clock/background_green", clk.props.green[0]);
-    gnome_config_set_int ("another_clock/background_blue", clk.props.blue[0]);
-    gnome_config_set_int ("another_clock/hourneedle_red", clk.props.red[1]);
-    gnome_config_set_int ("another_clock/hourneedle_green", clk.props.green[1]);
-    gnome_config_set_int ("another_clock/hourneedle_blue", clk.props.blue[1]);
-    gnome_config_set_int ("another_clock/minneedle_red", clk.props.red[2]);
-    gnome_config_set_int ("another_clock/minneedle_green", clk.props.green[2]);
-    gnome_config_set_int ("another_clock/minneedle_blue", clk.props.blue[2]);
-    gnome_config_set_int ("another_clock/secneedle_red", clk.props.red[3]);
-    gnome_config_set_int ("another_clock/secneedle_green", clk.props.green[3]);
-    gnome_config_set_int ("another_clock/secneedle_blue", clk.props.blue[3]);
+    gnome_config_set_bool ("another_clock/sec_needle", clk.props.secneedle);
+    gnome_config_set_string ("another_clock/background", clk.props.bg);
+    gnome_config_set_string ("another_clock/hourneedle", clk.props.hour);
+    gnome_config_set_string ("another_clock/minneedle", clk.props.min);
+    gnome_config_set_string ("another_clock/secneedle", clk.props.sec);
     gnome_config_sync();
     gnome_config_drop_all();
     gnome_config_pop_prefix();
@@ -116,15 +101,7 @@ int properties_save (char *path)
 
 void props_ok (GtkWidget *wid, int page, gpointer *data)
 {
-    int i;
-    
-    clk.props.sec = props_tmp.sec;
-    for (i = 0; i < 4; i++)
-    {
-	clk.props.red[i] = props_tmp.red[i];
-	clk.props.green[i] = props_tmp.green[i];
-	clk.props.blue[i] = props_tmp.blue[i];
-    } 
+    memcpy (&clk.props, &props_tmp, sizeof(struct clock_props_t));
     applet_widget_sync_config (APPLET_WIDGET(applet));
     set_colors(clk.area);
     update_clock (NULL);
@@ -139,52 +116,56 @@ void props_cancel (GtkWidget *widget, GtkWidget **win)
 
 void bg_color_changed (GnomeColorPicker *cp)
 {
-    gushort	a;
-    
-    gnome_color_picker_get_i16 (cp,
-				&props_tmp.red[0], &props_tmp.green[0],
-				&props_tmp.blue[0], &a);
+    guint8 r, g, b;
+    gchar  buf[24];
+		
+    gnome_color_picker_get_i8 (cp, &r, &g, &b, NULL);
+    sprintf (buf, "#%02x%02x%02x", r, g, b);
+    strcpy (props_tmp.bg, buf);
     gnome_property_box_changed (GNOME_PROPERTY_BOX(props_window));
 }
 
 
 void hour_color_changed (GnomeColorPicker *cp)
 {
-    gushort	a;
-
-    gnome_color_picker_get_i16 (cp,
-				&props_tmp.red[1], &props_tmp.green[1],
-				&props_tmp.blue[1], &a);
+    guint8 r, g, b;
+    gchar  buf[24];
+		
+    gnome_color_picker_get_i8 (cp, &r, &g, &b, NULL);
+    sprintf (buf, "#%02x%02x%02x", r, g, b);
+    strcpy (props_tmp.hour, buf);
     gnome_property_box_changed (GNOME_PROPERTY_BOX(props_window));
 }
 
 
 void min_color_changed (GnomeColorPicker *cp)
 {
-    gushort	a;
-
-    gnome_color_picker_get_i16 (cp,
-				&props_tmp.red[2], &props_tmp.green[2],
-				&props_tmp.blue[2], &a);
+    guint8 r, g, b;
+    gchar  buf[24];
+		
+    gnome_color_picker_get_i8 (cp, &r, &g, &b, NULL);
+    sprintf (buf, "#%02x%02x%02x", r, g, b);
+    strcpy (props_tmp.min, buf);
     gnome_property_box_changed (GNOME_PROPERTY_BOX(props_window));
 }
 
 
 void sec_color_changed (GnomeColorPicker *cp)
 {
-    gushort	a;
-
-    gnome_color_picker_get_i16 (cp,
-				&props_tmp.red[3], &props_tmp.green[3],
-				&props_tmp.blue[3], &a);
+    guint8 r, g, b;
+    gchar  buf[24];
+		
+    gnome_color_picker_get_i8 (cp, &r, &g, &b, NULL);
+    sprintf (buf, "#%02x%02x%02x", r, g, b);
+    strcpy (props_tmp.sec, buf);
     gnome_property_box_changed (GNOME_PROPERTY_BOX(props_window));
 }
 
 
 void sec_needle_changed (GtkWidget *widget, GtkWidget **sec)
 {
-    props_tmp.sec = props_tmp.sec ? FALSE : TRUE;
-    gtk_widget_set_sensitive (GTK_WIDGET(sec), props_tmp.sec);
+    props_tmp.secneedle = props_tmp.secneedle ? FALSE : TRUE;
+    gtk_widget_set_sensitive (GTK_WIDGET(sec), props_tmp.secneedle);
     gnome_property_box_changed (GNOME_PROPERTY_BOX(props_window));
 }
 
@@ -201,17 +182,14 @@ void cb_properties (AppletWidget *applet, gpointer data)
     GtkWidget *table;
     GtkWidget *sec_checkbox;
     GtkWidget *colorpicker;
-    int	      i;
+    guint     r, g, b;
 
     /* init temporal properties */
-    props_tmp.sec = clk.props.sec;
-    for (i = 0; i < 4; i++)
-    {
-	props_tmp.red[i] = clk.props.red[i];
-	props_tmp.green[i] = clk.props.green[i];
-	props_tmp.blue[i] = clk.props.blue[i];
-    } 
-
+    props_tmp.secneedle = clk.props.secneedle;
+    props_tmp.bg = g_strdup (clk.props.bg);
+    props_tmp.hour = g_strdup (clk.props.hour);
+    props_tmp.min = g_strdup (clk.props.min);
+    props_tmp.sec = g_strdup (clk.props.sec);
 
     help_entry.name = gnome_app_id;
 
@@ -259,9 +237,9 @@ void cb_properties (AppletWidget *applet, gpointer data)
 gtk_widget_set_sensitive (label, FALSE);
     gtk_widget_show (label);
     colorpicker = gnome_color_picker_new();
-    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(colorpicker),
-			        props_tmp.red[0], props_tmp.green[0],
-			        props_tmp.blue[0], 65535);
+    sscanf( clk.props.bg, "#%02x%02x%02x", &r, &g, &b);
+    gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(colorpicker),
+			        r, g, b, 255);
     gtk_table_attach_defaults (GTK_TABLE(table), colorpicker, 1, 2, 0, 1);
     gtk_signal_connect (GTK_OBJECT(colorpicker), "color_set",
 			GTK_SIGNAL_FUNC(bg_color_changed), NULL);
@@ -272,9 +250,9 @@ gtk_widget_set_sensitive (colorpicker, FALSE);
     gtk_table_attach_defaults (GTK_TABLE(table), label, 2, 3, 0, 1);
     gtk_widget_show (label);
     colorpicker = gnome_color_picker_new();
-    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(colorpicker),
-			        props_tmp.red[1], props_tmp.green[1],
-			        props_tmp.blue[1], 65535);
+    sscanf( clk.props.hour, "#%02x%02x%02x", &r, &g, &b);
+    gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(colorpicker),
+			        r, g, b, 255);
     gtk_table_attach_defaults (GTK_TABLE(table), colorpicker, 3, 4, 0, 1);
     gtk_signal_connect (GTK_OBJECT(colorpicker), "color_set",
 			GTK_SIGNAL_FUNC(hour_color_changed), NULL);
@@ -284,9 +262,9 @@ gtk_widget_set_sensitive (colorpicker, FALSE);
     gtk_table_attach_defaults (GTK_TABLE(table), label, 0, 1, 1, 2);
     gtk_widget_show (label);
     colorpicker = gnome_color_picker_new();
-    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(colorpicker),
-			        props_tmp.red[2], props_tmp.green[2],
-			        props_tmp.blue[2], 65535);
+    sscanf( clk.props.min, "#%02x%02x%02x", &r, &g, &b);
+    gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(colorpicker),
+			        r, g, b, 255);
     gtk_table_attach_defaults (GTK_TABLE(table), colorpicker, 1, 2, 1, 2);
     gtk_signal_connect (GTK_OBJECT(colorpicker), "color_set",
 			GTK_SIGNAL_FUNC(min_color_changed), NULL);
@@ -294,14 +272,14 @@ gtk_widget_set_sensitive (colorpicker, FALSE);
 
     label = gtk_label_new (_("Second needle color"));
     gtk_table_attach_defaults (GTK_TABLE(table), label, 2, 3, 1, 2);
-/*    gtk_widget_set_sensitive (label, props_tmp.sec);*/
+/*    gtk_widget_set_sensitive (label, clk.props.secneedle);*/
     gtk_widget_show (label);
     colorpicker = gnome_color_picker_new();
-    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(colorpicker),
-			        props_tmp.red[3], props_tmp.green[3],
-			        props_tmp.blue[3], 65535);
+    sscanf( clk.props.sec, "#%02x%02x%02x", &r, &g, &b);
+    gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(colorpicker),
+			        r, g, b, 255);
     gtk_table_attach_defaults (GTK_TABLE(table), colorpicker, 3, 4, 1, 2);
-    gtk_widget_set_sensitive (colorpicker, props_tmp.sec);
+    gtk_widget_set_sensitive (colorpicker, clk.props.secneedle);
     gtk_signal_connect (GTK_OBJECT(colorpicker), "color_set",
 			GTK_SIGNAL_FUNC(sec_color_changed), NULL);
     gtk_widget_show (colorpicker);
@@ -310,7 +288,7 @@ gtk_widget_set_sensitive (colorpicker, FALSE);
     /* second needle visible? */
     sec_checkbox = gtk_check_button_new_with_label (_("Show seconds needle"));
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(sec_checkbox),
-				  clk.props.sec);
+				  clk.props.secneedle);
     gtk_signal_connect (GTK_OBJECT(sec_checkbox), "toggled",
 			GTK_SIGNAL_FUNC(sec_needle_changed), colorpicker);
     gtk_box_pack_start (GTK_BOX(page), sec_checkbox, FALSE, FALSE, 0);
@@ -332,7 +310,7 @@ void cb_about (AppletWidget *applet, gpointer data)
 			    NULL
     };
 
-    about = gnome_about_new (_("Another Clock Applet"), "0.1",
+    about = gnome_about_new (_("Another Clock Applet"), "0.2",
 			     "(C) 1999 the Free Software Foundation",
                     	     authors,
                     	     _("An analog clock similar to that in CDE panel."),
@@ -352,35 +330,140 @@ static gint save_session (GtkWidget *widget, char *privcfgpath,
 }
 
 
+/**************************************************
+ * Stuff to mantain background under clock pixmap *
+ * Taken from gen_util/printer applet		  *
+ **************************************************/
+static void applet_set_default_back (GtkWidget *w)
+{
+    GtkStyle *ns;
+	
+    ns = gtk_style_new();
+    gtk_style_ref (ns);
+    gtk_widget_set_style (w, ns);
+    gtk_style_unref (ns);
+    gtk_widget_queue_draw (w);
+}
+
+
+static void applet_set_back_color (GtkWidget *w, GdkColor *color)
+{
+    GtkStyle *ns;
+	
+    ns = gtk_style_copy (w->style);
+    gtk_style_ref (ns);
+    ns->bg[GTK_STATE_NORMAL] = *color;
+    ns->bg[GTK_STATE_NORMAL].pixel = 1; /* bogus */
+					
+    if (ns->bg_pixmap[GTK_STATE_NORMAL])
+    {
+        gdk_imlib_free_pixmap (ns->bg_pixmap[GTK_STATE_NORMAL]);
+	ns->bg_pixmap[GTK_STATE_NORMAL] = NULL;
+    }
+
+    gtk_widget_set_style (w, ns);
+    gtk_style_unref (ns);
+    gtk_widget_queue_draw (w);
+}
+
+
+static void applet_set_back_pixmap (GtkWidget *w, gchar *pixmap)
+{
+    GdkImlibImage *im;
+    GdkPixmap 	  *p;
+    GtkStyle 	  *ns;
+
+    if (!pixmap || strcmp(pixmap,"") ==0 )
+    {
+        ns = gtk_style_copy (w->style);
+        gtk_style_ref (ns);
+        p = ns->bg_pixmap[GTK_STATE_NORMAL];
+        if (p)
+            gdk_imlib_free_pixmap (p);
+        ns->bg_pixmap[GTK_STATE_NORMAL] = NULL;
+        gtk_widget_set_style (w, ns);
+        gtk_style_unref (ns);
+        return;
+    }
+
+    if (!g_file_exists(pixmap))
+        return;
+			
+    im = gdk_imlib_load_image (pixmap);
+    if (!im)
+        return;
+
+    gdk_imlib_render (im, im->rgb_width, im->rgb_height);
+    p = gdk_imlib_move_image (im);
+    ns = gtk_style_copy (w->style);
+    gtk_style_ref (ns);
+    if(ns->bg_pixmap[GTK_STATE_NORMAL])
+        gdk_imlib_free_pixmap (ns->bg_pixmap[GTK_STATE_NORMAL]);
+    ns->bg_pixmap[GTK_STATE_NORMAL] = p;
+    gtk_widget_set_style (w, ns);
+
+    gtk_style_unref (ns);
+    gdk_imlib_destroy_image (im);
+}
+			
+
+static void applet_back_change (GtkWidget *widget, PanelBackType type,
+			        gchar *pixmap, GdkColor *color,
+				gpointer data)
+{
+    GtkWidget *w = data;
+
+    if (type == PANEL_BACK_PIXMAP)
+        applet_set_back_pixmap (w, pixmap);
+    else if (type == PANEL_BACK_COLOR)
+        applet_set_back_color (w, color);
+    else
+        applet_set_default_back (w);
+}
+
+
 /**********
  * Colors *
  **********/
 void set_gc_color(GdkColorContext *cc, int n)
 {
-    GdkColor        *c;
-    gint	    z;
+    GdkColor    *c;
+    gint	z;
+    gchar	*color;
+    guint	r, g, b;
+    
+    switch (n)
+    {
+	case 0:	color = clk.props.bg; break;
+	case 1:	color = clk.props.hour; break;
+	case 2:	color = clk.props.min; break;
+	case 3:	color = clk.props.sec; break;
+	default:	color = clk.props.bg; break;
+    }
 
     c = g_new (GdkColor, 1);
-    c->red = clk.props.red[n];
-    c->green = clk.props.green[n];
-    c->blue = clk.props.blue[n];
+    sscanf( color, "#%02x%02x%02x", &r ,&g,&b);
+
+    c->red = (gulong) r * 256;
+    c->green = (gulong) g * 256;
+    c->blue = (gulong) b * 256;
     c->pixel = (gulong) 0;
     z = 0;
     gdk_color_context_get_pixels (cc, &c->red, &c->green, &c->blue, 1,
 	                          &c->pixel, &z);
     if (n == 0)
     {
-       /* Next doesn't work! Why????????? */
-/*       gdk_window_set_background (clk.clock_xpm, c);*/
+       /* Here it would be code to change a color (background, not
+          transparent) in a pixmap */
     }
     else
     {
-       gdk_gc_set_foreground (clk.gc[n], c);
-       gdk_gc_set_line_attributes (clk.gc[n],
-				   (n == 1) ? 2 : 1,
-				   GDK_LINE_SOLID,
-	                	   GDK_CAP_ROUND,
-				   GDK_JOIN_ROUND);
+        gdk_gc_set_foreground (clk.gc[n], c);
+        gdk_gc_set_line_attributes (clk.gc[n],
+				    (n == 1) ? 2 : 1,
+				    GDK_LINE_SOLID,
+	                	    GDK_CAP_ROUND,
+				    GDK_JOIN_ROUND);
     }
     g_free (c);
 }
@@ -422,7 +505,7 @@ static gint update_clock (gpointer data)
     /* draw clock */
     gdk_draw_pixmap (clk.xpm,
 	             clk.gc[0],
-                     clk.clock_xpm,
+                     GNOME_PIXMAP(clk.clock)->pixmap,
                      0, 0, 0, 0,
 		     APPLET_WIDTH, APPLET_HEIGHT);
 
@@ -436,7 +519,7 @@ static gint update_clock (gpointer data)
     gdk_draw_line (clk.xpm, clk.gc[2], CX, CY,
 		   (int) (CX + NEEDLE_SIZE * sin(ang)),
 		   (int) (CY - NEEDLE_SIZE * cos(ang)));
-    if (clk.props.sec)
+    if (clk.props.secneedle)
     {
        ang = tm->tm_sec * M_PI / 30;
        gdk_draw_line (clk.xpm, clk.gc[3], CX, CY,
@@ -448,9 +531,7 @@ static gint update_clock (gpointer data)
     r.y = 0;
     r.width = APPLET_WIDTH;
     r.height = APPLET_HEIGHT;
-
-    gdk_window_set_back_pixmap (clk.area->window, clk.xpm, FALSE);
-    gdk_window_clear (clk.area->window);
+    GNOME_PIXMAP(clk.pixmap)->pixmap = clk.xpm;
     gtk_widget_draw (clk.area, &r);
 
     return TRUE;
@@ -474,43 +555,49 @@ int main (int argc, char *argv[])
     applet_widget_init (APPLET_NAME, VERSION, argc, argv, NULL, 0, NULL);
     applet = applet_widget_new (APPLET_NAME);
     if (!applet)
-       g_error ("Can't create another_clock applet");
+       g_error ("Can't create another_clock applet!\n");
+    gtk_widget_realize (applet);
 
-    /* create drawing area for clock pixmap */
-    clk.area = gtk_drawing_area_new();
-    gtk_drawing_area_size (GTK_DRAWING_AREA(clk.area),
-			   APPLET_WIDTH, APPLET_HEIGHT);
-    gtk_widget_show (clk.area);
+    /* create fixed area for clock pixmap */
+    clk.area = gtk_fixed_new();
     gtk_widget_set_usize (clk.area, APPLET_WIDTH, APPLET_HEIGHT);
-    applet_widget_add (APPLET_WIDGET(applet), clk.area);
-    gtk_widget_realize (clk.area);
 
     /* load clock pixmap  */
     fname = gnome_unconditional_pixmap_file (CLOCK_XPM_FILE);
     if (!fname)
 	g_error ("Can't find another_clock applet pixmap");
-    clk.clock_xpm = gdk_pixmap_create_from_xpm (clk.area->window,
-						&clk.clock_mask,
-                                                &clk.area->style->bg[GTK_STATE_NORMAL],
-                                                fname);
-    
+    clk.clock = gnome_pixmap_new_from_file_at_size (fname,
+						    APPLET_WIDTH,
+						    APPLET_HEIGHT);
+    g_free (fname);
+    gtk_widget_show (clk.clock);
+
     /* create a pixmap for clock */
-    clk.xpm = gdk_pixmap_new (clk.area->window, APPLET_WIDTH, APPLET_HEIGHT, -1);
+    clk.pixmap = gnome_pixmap_new_from_gnome_pixmap (GNOME_PIXMAP(clk.clock));
+    gtk_fixed_put (GTK_FIXED(clk.area), clk.pixmap, 0, 0);
+    gtk_widget_show (clk.pixmap);
+    gtk_widget_show (clk.area);
+
+    /* create a gdk_pixmap for buffer operations */
+    clk.xpm = gdk_pixmap_new (applet->window, APPLET_WIDTH, APPLET_HEIGHT, -1);
 
     /* load background and needles' colors */
     properties_load (APPLET_WIDGET(applet)->privcfgpath);
-    set_colors (clk.area);
-
-    /* show applet */
-    gtk_widget_show(applet);
+    set_colors (applet);
 
     /* callback for updating the time */
-    gtk_timeout_add (100, update_clock, &clock);
-    update_clock(&clock);
+    gtk_timeout_add (500, update_clock, &clk);
+    update_clock (&clk);
 
-    /* callbacks for session, about, properties, etc. */
+    /* show applet */
+    applet_widget_add (APPLET_WIDGET(applet), clk.area);
+    gtk_widget_show (applet);
+
+    /* callbacks for session, background change, about, properties, etc. */
     gtk_signal_connect (GTK_OBJECT(applet), "save_session",
 			GTK_SIGNAL_FUNC(save_session), NULL);
+    gtk_signal_connect (GTK_OBJECT(applet), "back_change",
+	                GTK_SIGNAL_FUNC(applet_back_change), clk.area);
     applet_widget_register_stock_callback (APPLET_WIDGET(applet),
 					   "properties",
 					   GNOME_STOCK_MENU_PROP,
