@@ -25,9 +25,14 @@
 static void about_cb (AppletWidget *widget, gpointer data);
 static void set_tooltip(struct tm *time_data, AppData *ad);
 static void redraw_display(AppData *ad);
-static void update_mail_display(int n, AppData *ad);
+static void update_mail_display(int n, AppData *ad, gint force);
+static void update_mail_count(AppData *ad, gint force);
+static void update_mail_amount_display(AppData *ad, gint force);
 static void update_time_count(gint h, gint m, gint s, AppData *ad);
+static void update_date_displays(gint year, gint month, gint day, gint weekday, AppData *ad, gint force);
+static void update_time_and_date(AppData *ad, gint force);
 static gint blink_callback(gpointer data);
+static void update_mail_status(AppData *ad, gint force);
 static gint update_display(gpointer data);
 static void applet_change_back(GtkWidget *applet, PanelBackType type, char *pixmap,
 				GdkColor *color, gpointer data);
@@ -63,6 +68,13 @@ static void about_cb (AppletWidget *widget, gpointer data)
 	gtk_widget_show (about);
 }
 
+void launch_mail_reader(gpointer data)
+{
+	AppData *ad = data;
+	if (ad->reader_exec_cmd && strlen(ad->reader_exec_cmd) > 0)
+		gnome_execute_shell(NULL, ad->reader_exec_cmd);
+}
+
 /*
  * Get file modification time, based upon the code
  * of Byron C. Darrah for coolmail and reused on fvwm95
@@ -104,7 +116,7 @@ void check_mail_file_status (int reset, AppData *ad)
 	ad->unreadmail = (s.st_mtime >= s.st_atime && newsize > 0);
 
 	checktime = (newtime > ad->oldtime);
-	if (newsize >= ad->mailsize && ad->unreadmail && checktime){
+	if (newsize > ad->mailsize && ad->unreadmail && checktime){
 		ad->newmail = 1;
 		ad->mailcleared = 0;
 	} else
@@ -118,7 +130,7 @@ static void set_tooltip(struct tm *time_data, AppData *ad)
 	if (time_data->tm_yday != ad->old_yday)
 		{
 		gchar date[128];
-		strftime(date, 128, "%a, %b %d", time_data);
+		strftime(date, 128, _("%a, %b %d"), time_data);
 
 		if (ad->use_gmt)
 			{
@@ -148,8 +160,13 @@ static void redraw_display(AppData *ad)
 	redraw_skin(ad);
 }
 
-static void update_mail_display(int n, AppData *ad)
+static void update_mail_display(int n, AppData *ad, gint force)
 {
+	if (force)
+		{
+		draw_item(ad->skin->mail, ad->old_n, ad);
+		return;
+		}
 	if (n != ad->old_n)
 		{
 		draw_item(ad->skin->mail, n, ad);
@@ -157,9 +174,44 @@ static void update_mail_display(int n, AppData *ad)
 		}
 }
 
-static void update_mail_amount_display(AppData *ad)
+static void update_mail_count(AppData *ad, gint force)
 {
-	if (ad->mailsize != ad->old_amount)
+	if (!ad->skin->messages) return;
+
+	if (force)
+		{
+		draw_number(ad->skin->messages, ad->message_count, ad);
+		return;
+		}
+
+	if (ad->mail_file)
+		{
+		FILE *f = 0;
+		gchar buf[1024];
+
+		f = fopen(ad->mail_file, "r");
+
+		if (f)
+			{
+			gint c = 0;
+			while (fgets(buf, sizeof(buf), f) != NULL)
+				{
+				if (buf[0] == 'F' && !strncmp(buf, "From:", 5)) c++;
+				}
+                        fclose(f);
+			ad->message_count = c;
+                        }
+		else
+			{
+			ad->message_count = 0;
+			}
+		}
+	draw_number(ad->skin->messages, ad->message_count, ad);
+}
+
+static void update_mail_amount_display(AppData *ad, gint force)
+{
+	if (ad->mailsize != ad->old_amount || force)
 		{
 		if (ad->skin->mail_amount)
 			{
@@ -173,6 +225,7 @@ static void update_mail_amount_display(AppData *ad)
 			}
 		draw_number(ad->skin->mail_count, ad->mailsize / 1024, ad);
 		ad->old_amount = ad->mailsize;
+		update_mail_count(ad, force);
 		}
 }
 
@@ -188,10 +241,10 @@ static void update_time_count(gint h, gint m, gint s, AppData *ad)
 	draw_number(ad->skin->sec, s, ad);
 }
 
-static void update_date_displays(gint year, gint month, gint day, gint weekday, AppData *ad)
+static void update_date_displays(gint year, gint month, gint day, gint weekday, AppData *ad, gint force)
 {
 	/* no point in drawing things again if they don't change */
-	if (ad->old_week == weekday) return;
+	if (ad->old_week == weekday && !force) return;
 
 	draw_number(ad->skin->year, year, ad);
 	draw_number(ad->skin->month, month, ad);
@@ -202,46 +255,8 @@ static void update_date_displays(gint year, gint month, gint day, gint weekday,
 	ad->old_week = weekday;
 }
 
-static gint blink_callback(gpointer data)
+static void update_time_and_date(AppData *ad, gint force)
 {
-	AppData *ad = data;
-	ad->blink_lit++;
-
-	if (ad->blink_lit >= ad->mail_sections - 1)
-		{
-		ad->blink_lit = 0;
-		ad->blink_count++;
-		}
-	update_mail_display(ad->blink_lit, ad);
-	redraw_display(ad);
-
-	if (ad->blink_count >= ad->blink_times || !ad->anymail)
-		{
-		if (ad->always_blink && ad->anymail)
-			{
-			if (ad->blink_count >= ad->blink_times) ad->blink_count = 1;
-			}
-		else
-			{
-			/* reset counters for next time */
-			ad->blink_count = 0;
-			ad->blink_lit = 0;
-			if (ad->anymail)
-				update_mail_display(ad->mail_sections - 1, ad);
-			else
-				update_mail_display(0, ad);
-			redraw_display(ad);
-			ad->blinking = FALSE;
-			return FALSE;
-			}
-		}
-
-	return TRUE;
-}
-
-static gint update_display(gpointer data)
-{
-	AppData *ad = data;
 	time_t current_time;
 	struct tm *time_data;
 
@@ -261,12 +276,57 @@ static gint update_display(gpointer data)
 
 	/* update date */
 	update_date_displays(time_data->tm_year, time_data->tm_mon, time_data->tm_mday,
-				time_data->tm_wday, ad);
+				time_data->tm_wday, ad, force);
 
 	/* set tooltip to the date */
 	set_tooltip(time_data, ad);
+}
 
-	/* now check mail */
+static gint blink_callback(gpointer data)
+{
+	AppData *ad = data;
+	ad->blink_lit++;
+
+	if (ad->blink_lit >= ad->mail_sections - 1)
+		{
+		ad->blink_lit = 0;
+		ad->blink_count++;
+		}
+	update_mail_display(ad->blink_lit, ad, FALSE);
+	redraw_display(ad);
+
+	if (ad->blink_count >= ad->blink_times || !ad->anymail)
+		{
+		if (ad->always_blink && ad->anymail)
+			{
+			if (ad->blink_count >= ad->blink_times) ad->blink_count = 1;
+			}
+		else
+			{
+			/* reset counters for next time */
+			ad->blink_count = 0;
+			ad->blink_lit = 0;
+			if (ad->anymail)
+				update_mail_display(ad->mail_sections - 1, ad, FALSE);
+			else
+				update_mail_display(0, ad, FALSE);
+			redraw_display(ad);
+			ad->blinking = FALSE;
+			return FALSE;
+			}
+		}
+
+	return TRUE;
+}
+
+static void update_mail_status(AppData *ad, gint force)
+{
+	if (force)
+		{
+		update_mail_amount_display(ad, TRUE);
+		update_mail_display(0, ad, TRUE);
+		return;
+		}
 	check_mail_file_status (FALSE, ad);
 
 	if (!ad->blinking)
@@ -285,16 +345,30 @@ static gint update_display(gpointer data)
 				}
 			else
 				{
-				update_mail_display(ad->mail_sections - 1, ad);
+				update_mail_display(ad->mail_sections - 1, ad, FALSE);
 				}
 		else
 			{
-			update_mail_display(0, ad);
+			update_mail_display(0, ad, FALSE);
 			}
 		}
 
-	update_mail_amount_display(ad);
+	update_mail_amount_display(ad, FALSE);
+}
 
+void redraw_all(gpointer data)
+{
+	AppData *ad = data;
+	update_time_and_date(ad, TRUE);
+	update_mail_status(ad, TRUE);
+}
+
+static gint update_display(gpointer data)
+{
+	AppData *ad = data;
+
+	update_time_and_date(ad, FALSE);
+	update_mail_status(ad, FALSE);
 	redraw_display(ad);
 
 	return TRUE;
@@ -306,7 +380,6 @@ static void applet_change_back(GtkWidget *applet, PanelBackType type, char *pixm
 	AppData *ad = data;
 	GtkStyle *ns;
 
-	printf("back changes\n");
 	switch (type)
 		{
 		case PANEL_BACK_NONE :
@@ -380,6 +453,8 @@ static AppData *create_new_app(GtkWidget *applet)
 	ad->gmt_offset = 0;
 	ad->mail_max = 150;
 
+	ad->message_count = 0;
+
 	/* (duration = BLINK_DELAY / 1000 * BLINK_TIMES) */
 	ad->blink_delay = 166;
 	ad->blink_times = 16;
@@ -390,6 +465,9 @@ static AppData *create_new_app(GtkWidget *applet)
 	/* mail file location */
 	ad->mail_file = NULL;
 
+	/* execute a command on button press */
+	ad->reader_exec_cmd = NULL;
+
 	/* execute a command on new mail */
 	ad->newmail_exec_cmd = NULL;
 	ad->exec_cmd_on_newmail = FALSE;
@@ -424,6 +502,7 @@ static AppData *create_new_app(GtkWidget *applet)
 
 	applet_widget_add(APPLET_WIDGET(ad->applet), ad->display_area);
 
+	skin_event_init(ad);
 	gtk_widget_realize(ad->display_area);
 
 	if (ad->theme_file && strlen(ad->theme_file) == 0)
