@@ -299,6 +299,8 @@ properties_dialog(AppletWidget *aw, gpointer data)
 static void 
 update_fortune_dialog(Fish *fish)
 {
+        gboolean fortune_exists;
+
 	if ( fish->fortune_dialog == NULL ) {
 		fish->fortune_dialog = 
 			gnome_dialog_new("", GNOME_STOCK_BUTTON_CLOSE, NULL);
@@ -310,6 +312,20 @@ update_fortune_dialog(Fish *fish)
 		fish->fortune_less = gnome_less_new();
 		fish->fortune_label = gtk_label_new("");
 
+		gnome_less_set_fixed_font(GNOME_LESS(fish->fortune_less),TRUE);
+		
+		{
+			int i;
+			char buf[82];
+			GtkWidget *text = GTK_WIDGET(GNOME_LESS(fish->fortune_less)->text);
+			GdkFont *font = GNOME_LESS(fish->fortune_less)->font;
+			for(i=0;i<81;i++) buf[i]='X';
+			buf[i]='\0';
+			gtk_widget_set_usize(text,
+					     gdk_string_width(font,buf)+30,
+					     gdk_string_height(font,buf)*24+30);
+		}
+
 		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(fish->fortune_dialog)->vbox), 
 				   fish->fortune_label,
 				   FALSE, FALSE, GNOME_PAD);
@@ -318,17 +334,23 @@ update_fortune_dialog(Fish *fish)
 				   fish->fortune_less,
 				   TRUE, TRUE, GNOME_PAD);
 
-		gnome_less_set_fixed_font(GNOME_LESS(fish->fortune_less),TRUE);
-
 		gtk_widget_show(fish->fortune_less);
 		gtk_widget_show(fish->fortune_label);
 		apply_properties(fish);
 	}
 	if (!GTK_WIDGET_VISIBLE(fish->fortune_dialog))
-		gtk_widget_show(fish->fortune_dialog);  
-	gnome_less_show_command(GNOME_LESS(fish->fortune_less),
-				g_file_exists("/usr/games/fortune")?
-				"/usr/games/fortune":"fortune");
+		gtk_widget_show(fish->fortune_dialog);
+
+	fortune_exists = g_file_exists("/usr/games/fortune") || 
+	                 (gnome_is_program_in_path("fortune") != NULL);
+
+	if (fortune_exists)
+                gnome_less_show_command(GNOME_LESS(fish->fortune_less),
+					g_file_exists("/usr/games/fortune")?
+					"/usr/games/fortune":"fortune");
+	else
+                gnome_less_show_string(GNOME_LESS(fish->fortune_less),
+				       "You do not have fortune installed.");
 }
 
 static int 
