@@ -32,12 +32,14 @@ struct _ClockData {
 	int timeouttime;
 	int hourformat;
         int showdate;
+	int unixtime;
 	ClockUpdateFunc update_func;
 	PanelOrientType orient;
 
 	GtkWidget *props;
 	int prop_hourformat;
         int prop_showdate;
+	int prop_unixtime;
 };
 
 
@@ -60,18 +62,27 @@ clock_timeout_callback(gpointer data)
 {
 	ClockData *cd = data;
 	time_t current_time;
-
+	
 	time(&current_time);
 
 	(*cd->update_func) (cd, current_time);
 
-	if (cd->timeouttime != 36000)
-	{
-	   cd->timeouttime = 36000;
-	   cd->timeout = gtk_timeout_add(cd->timeouttime,
-			clock_timeout_callback,
-			cd);
-	   return FALSE;
+	if(cd->unixtime) {
+		if (cd->timeouttime != 1000) {
+			cd->timeouttime = 1000;
+			cd->timeout = gtk_timeout_add(cd->timeouttime,
+						      clock_timeout_callback,
+						      cd);
+			return FALSE;
+		}
+	} else {
+		if (cd->timeouttime != 36000) {
+			cd->timeouttime = 36000;
+			cd->timeout = gtk_timeout_add(cd->timeouttime,
+						      clock_timeout_callback,
+						      cd);
+			return FALSE;
+		}
 	}
 
 	return TRUE;
@@ -87,6 +98,7 @@ applet_save_session(GtkWidget * w,
 	gnome_config_push_prefix(privcfgpath);
 	gnome_config_set_int("clock/hourformat", cd->hourformat);
 	gnome_config_set_int("clock/showdate", cd->showdate);
+	gnome_config_set_int("clock/unixtime", cd->unixtime);
 	gnome_config_pop_prefix();
 	gnome_config_sync();
 	gnome_config_drop_all();
@@ -106,7 +118,7 @@ computer_clock_update_func(ClockData * cd, time_t current_time)
 
 	tm = localtime(&current_time);
 
-	if (cd->showdate) {
+	if (cd->showdate && !cd->unixtime) {
 	    if (cd->orient == ORIENT_LEFT || cd->orient == ORIENT_RIGHT) {
 	        /* This format string is used, to display the actual day,
 		   when showing a vertical panel.  For an explanation of
@@ -127,7 +139,15 @@ computer_clock_update_func(ClockData * cd, time_t current_time)
 	          gtk_widget_hide (cc->date);
 	  }
 
-	if (cd->hourformat == 0) {
+	if (cd->unixtime) {
+	    if (cd->orient == ORIENT_LEFT || cd->orient == ORIENT_RIGHT) {
+		    g_snprintf(hour,20,"%lu\n%lu",
+			       (unsigned long)(current_time/100000L),
+			       (unsigned long)(current_time%100000L));
+	    } else {
+		    g_snprintf(hour,20,"%lu",(unsigned long)current_time);
+	    } 
+	} else if (cd->hourformat == 0) {
 	    /* This format string is used, to display the actual time in
 	       12 hour format.  */
 	    if (cd->orient == ORIENT_LEFT || cd->orient == ORIENT_RIGHT) {
@@ -193,16 +213,13 @@ destroy_clock(GtkWidget * widget, void *data)
 	g_free(cd);
 }
 
-static ClockData *
-create_clock_widget(GtkWidget * applet)
+static void
+create_clock_widget(ClockData *cd, GtkWidget * applet)
 {
 	GtkWidget *clock;
-	ClockData *cd;
 	struct tm *tm;
 	time_t current_time;
 
-	cd = g_new(ClockData, 1);
-
 	/*FIXME: different clock types here */
 	create_computer_clock_widget(&clock, &cd->update_func);
 
@@ -222,12 +239,13 @@ create_clock_widget(GtkWidget * applet)
 
 	/* Install timeout handler */
         tm = localtime(&current_time);
-	cd->timeouttime = 36000-(tm->tm_sec*600);
+	if(cd->unixtime)
+		cd->timeouttime = 1000;
+	else
+		cd->timeouttime = 36000-(tm->tm_sec*600);
 	cd->timeout = gtk_timeout_add(cd->timeouttime,
-			clock_timeout_callback,
-			cd);
-
-	return cd;
+				      clock_timeout_callback,
+				      cd);
 }
 
 /*these are commands sent over corba: */
@@ -254,13 +272,16 @@ make_clock_applet(const gchar * goad_id)
 	if (!applet)
 		g_error(_("Can't create applet!\n"));
 
-	cd = create_clock_widget(applet);
+	cd = g_new(ClockData, 1);
 
 	gnome_config_push_prefix(APPLET_WIDGET(applet)->privcfgpath);
 	cd->hourformat = gnome_config_get_int("clock/hourformat=0");
 	cd->showdate = gnome_config_get_int("clock/showdate=1");
+	cd->unixtime = gnome_config_get_int("clock/unixtime=0");
 	gnome_config_pop_prefix();
 
+	create_clock_widget(cd,applet);
+
 	/*we have to bind change_orient before we do applet_widget_add 
 	   since we need to get an initial change_orient signal to set our
 	   initial oriantation, and we get that during the _add call */
@@ -291,23 +312,36 @@ apply_properties(GtkWidget * widget, gint button_num, gpointer data)
 {
 	ClockData *cd = data;
 	time_t current_time;
+	struct tm *tm;
 
 	cd->hourformat = cd->prop_hourformat;
 	cd->showdate = cd->prop_showdate;
+	cd->unixtime = cd->prop_unixtime;
 
+	/* Call the clock's update function so that it paints its first state */
 	time(&current_time);
 	(*cd->update_func) (cd, current_time);
+	
+	gtk_timeout_remove(cd->timeout);
 
-	gtk_widget_queue_resize (cd->clockw);
+	/* Install timeout handler */
+        tm = localtime(&current_time);
+	if(cd->unixtime)
+		cd->timeouttime = 1000;
+	else
+		cd->timeouttime = 36000-(tm->tm_sec*600);
+	cd->timeout = gtk_timeout_add(cd->timeouttime,
+				      clock_timeout_callback,
+				      cd);
+/* gtk_widget_queue_resize (cd->clockw);*/
 }
 
-static int
+static void
 close_properties(GtkWidget * w, gpointer data)
 {
 	ClockData *cd = data;
 
 	cd->props = NULL;
-	return FALSE;
 }
 
 static void
@@ -330,17 +364,34 @@ set_show_date_cb(GtkWidget * w, gpointer data)
 	gnome_property_box_changed (GNOME_PROPERTY_BOX (cd->props));
 }
 
+static void
+set_datasensitive_cb(GtkWidget * w, GtkWidget *wid)
+{
+	gtk_widget_set_sensitive(wid,!(GTK_TOGGLE_BUTTON(w)->active));
+}
+
+
+static void
+set_unixtime_cb(GtkWidget * w, gpointer data)
+{
+	ClockData *cd = data;
+
+	cd->prop_unixtime = GTK_TOGGLE_BUTTON(w)->active;
+	gnome_property_box_changed (GNOME_PROPERTY_BOX (cd->props));
+}
 
 static void
 clock_properties(AppletWidget * applet, gpointer data)
 {
         static GnomeHelpMenuEntry help_entry = { NULL, "properties-clock" };
         GtkWidget *hbox;
-        GtkWidget *frame;
+        GtkWidget *vbox;
+        GtkWidget *hour_frame;
 	GtkWidget *table;
 	GtkWidget *twelvehour;
 	GtkWidget *twentyfourhour;
 	GtkWidget *showdate;
+	GtkWidget *unixtime;
 	ClockData *cd = data;
 
 	help_entry.name = gnome_app_id;
@@ -355,12 +406,12 @@ clock_properties(AppletWidget * applet, gpointer data)
 
 	hbox = gtk_hbox_new(FALSE, GNOME_PAD);
 
-	frame = gtk_frame_new(_("Time Format"));
-	gtk_container_set_border_width(GTK_CONTAINER(frame), GNOME_PAD);
-	gtk_box_pack_start (GTK_BOX(hbox), frame, FALSE, FALSE, 0);
+	hour_frame = gtk_frame_new(_("Time Format"));
+	gtk_container_set_border_width(GTK_CONTAINER(hour_frame), GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX(hbox), hour_frame, FALSE, FALSE, 0);
 
 	table = gtk_table_new(2, 2, FALSE);
-	gtk_container_add(GTK_CONTAINER(frame), table);
+	gtk_container_add(GTK_CONTAINER(hour_frame), table);
 	gtk_widget_show(table);
 
 	gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD);
@@ -406,9 +457,13 @@ clock_properties(AppletWidget * applet, gpointer data)
 			   "toggled",
 			   (GtkSignalFunc) set_hour_format_cb,
 			   (gpointer) 1);
+	
+	vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
+	gtk_box_pack_start_defaults(GTK_BOX(hbox), vbox);
+	gtk_widget_show(vbox);
 
 	showdate = gtk_check_button_new_with_label(_("Show date"));
-	gtk_box_pack_start_defaults(GTK_BOX(hbox), showdate);
+	gtk_box_pack_start_defaults(GTK_BOX(vbox), showdate);
 	gtk_widget_show(showdate);
 	
 	if (cd->showdate)
@@ -421,15 +476,32 @@ clock_properties(AppletWidget * applet, gpointer data)
 			   (GtkSignalFunc) set_show_date_cb,
 			   data);
 
-	gtk_widget_show(frame);
+	unixtime = gtk_check_button_new_with_label(_("Unix time"));
+	gtk_box_pack_start_defaults(GTK_BOX(vbox), unixtime);
+	gtk_widget_show(unixtime);
+
+	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   hour_frame);
+	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
+			   (GtkSignalFunc) set_datasensitive_cb,
+			   showdate);
+	
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(unixtime),
+				     cd->unixtime);
+	cd->prop_unixtime = cd->unixtime;
+	
+	gtk_signal_connect(GTK_OBJECT(unixtime), "toggled",
+			   (GtkSignalFunc) set_unixtime_cb,
+			   data);
+
+	gtk_widget_show(hour_frame);
 	gtk_widget_show(hbox);
 
 	gnome_property_box_append_page(GNOME_PROPERTY_BOX(cd->props), hbox,
 				       gtk_label_new(_("Clock")));
 	gtk_signal_connect(GTK_OBJECT(cd->props), "apply",
 			   GTK_SIGNAL_FUNC(apply_properties), data);
-	gtk_signal_connect(GTK_OBJECT(cd->props), "delete_event",
-			   GTK_SIGNAL_FUNC(close_properties), data);
 	gtk_signal_connect(GTK_OBJECT(cd->props), "destroy",
 			   GTK_SIGNAL_FUNC(close_properties), data);
 	gtk_signal_connect(GTK_OBJECT(cd->props), "help",
