@@ -30,19 +30,22 @@
 
 #define MODEMLIGHTS_APPLET_VERSION_MAJ 1
 #define MODEMLIGHTS_APPLET_VERSION_MIN 0
-#define MODEMLIGHTS_APPLET_VERSION_REV 1
+#define MODEMLIGHTS_APPLET_VERSION_REV 2
 
 extern gint UPDATE_DELAY;
 extern gchar *lock_file;
+extern gint verify_lock_file;
 extern gchar *command_connect;
 extern gchar *command_disconnect;
 extern int ask_for_confirmation;
 extern gchar *device_name;
 extern gint use_ISDN;
+extern gint show_extra_info;
 
 extern GtkWidget *applet;
 
 void start_callback_update(void);
+void reset_orientation(void);
 
 void property_load(char *path);
 void property_save(char *path);
