@@ -55,8 +55,8 @@ static IceAuthFileEntry* file_entry_new (const gchar* protocol,
 static GSList* read_authfile(gchar* filename);
 static void write_authfile(gchar* filename, GSList* entries);
 
-/* This is called when a client tries to connect.  We accept most
-   connections.  We reject connections while doing a shutdown.  */
+/* This is called when a client tries to connect. 
+ * We still accept connections during a shutdown so we can handle warnings. */
 static void
 accept_connection (gpointer client_data, gint source,
 		   GdkInputCondition conditon)
@@ -65,9 +65,6 @@ accept_connection (gpointer client_data, gint source,
   IceAcceptStatus status;
   IceConnectStatus status2 = IceConnectPending;
 
-  if (shutdown_in_progress_p ())
-    return;
-
   connection = IceAcceptConnection ((IceListenObj) client_data, &status);
   if (status != IceAcceptSuccess)
     return;
@@ -184,7 +181,6 @@ initialize_ice (void)
 
   p = g_strconcat (ENVNAME "=", ids, NULL);
   putenv (p);
-  g_free (p);
 }
 
 void
