@@ -28,7 +28,7 @@ int applet_count = 0;
 
 /*config sync stuff*/
 extern int config_sync_timeout;
-extern GSList *applets_to_sync;
+extern int applets_to_sync;
 extern int panels_to_sync;
 extern int globals_to_sync;
 extern int need_complete_save;
@@ -122,6 +122,8 @@ applet_menu_deactivate(GtkWidget *w, AppletInfo *info)
 	
 	if(IS_SNAPPED_WIDGET(panel))
 		SNAPPED_WIDGET(panel)->autohide_inhibit = FALSE;
+	else if (IS_CORNER_WIDGET(panel))
+	        CORNER_WIDGET(panel)->autohide_inhibit = FALSE;
 }
 
 static AppletUserMenu *
@@ -342,6 +344,9 @@ show_applet_menu(AppletInfo *info, GdkEventButton *event)
 	if(IS_SNAPPED_WIDGET(panel)) {
 		SNAPPED_WIDGET(panel)->autohide_inhibit = TRUE;
 		snapped_widget_queue_pop_down(SNAPPED_WIDGET(panel));
+	} else if (IS_CORNER_WIDGET(panel)) {
+	        CORNER_WIDGET(panel)->autohide_inhibit = TRUE;
+		corner_widget_queue_pop_down(CORNER_WIDGET(panel));
 	}
 	info->menu_age = 0;
 	gtk_menu_popup(GTK_MENU(info->menu), NULL, NULL,
@@ -470,8 +475,7 @@ register_toy(GtkWidget *applet,
 	applet_count++;
 
 	/*we will need to save this applet's config now*/
-	if(g_slist_find(applets_to_sync, info)==NULL)
-		applets_to_sync = g_slist_prepend(applets_to_sync,info);
+	applets_to_sync = TRUE;
 
 	if(panel_widget_add_full(panel, applet, pos, TRUE)==-1) {
 		GSList *list;
