@@ -27,7 +27,7 @@ extern GSList *applets_last;
 extern int applet_count;
 
 extern int config_sync_timeout;
-extern GSList *applets_to_sync;
+extern int applets_to_sync;
 extern int panels_to_sync;
 extern int globals_to_sync;
 extern int need_complete_save;
@@ -638,8 +638,11 @@ s_panelspot_show_menu(POA_GNOME_PanelSpot *servant,
 	if(IS_SNAPPED_WIDGET(panel)) {
 		SNAPPED_WIDGET(panel)->autohide_inhibit = TRUE;
 		snapped_widget_queue_pop_down(SNAPPED_WIDGET(panel));
+	} else if (IS_CORNER_WIDGET(panel)) {
+	        CORNER_WIDGET(panel)->autohide_inhibit = TRUE;
+	        corner_widget_queue_pop_down(CORNER_WIDGET(panel));
 	}
-
+	
 	ext->info->menu_age = 0;
 	gtk_menu_popup(GTK_MENU(ext->info->menu), NULL, NULL,
 		       global_config.off_panel_popups?applet_menu_position:NULL,
@@ -722,8 +725,7 @@ s_panelspot_sync_config(POA_GNOME_PanelSpot *servant,
 
 	g_return_if_fail(ext != NULL);
 	g_return_if_fail(ext->info != NULL);
-	if(g_slist_find(applets_to_sync, ext->info)==NULL)
-		applets_to_sync = g_slist_prepend(applets_to_sync,ext->info);
+	applets_to_sync = TRUE;
 	panel_config_sync();
 }
 
