@@ -14,6 +14,7 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <string.h>
+#include <limits.h>
 #include <errno.h>
 #include <gnome.h>
 
@@ -21,7 +22,7 @@
 #include "panel-widget.h"
 
 #define SMALL_ICON_SIZE 20
-#define BIG_ICON_SIZE   PANEL_MINIMUM_WIDTH
+#define BIG_ICON_SIZE   48
 
 
 #define MENU_PATH "menu_path"
@@ -244,7 +245,8 @@ fr_fill_dir(FileRec *fr, int sublevels)
 		} else {
 			GnomeDesktopEntry *dentry;
 			char *p = strrchr(name,'.');
-			if(p && strcmp(p,".desktop")!=0) {
+			if (!p || (strcmp(p, ".desktop") != 0 &&
+				   strcmp(p, ".kdelnk") != 0)) {
 				g_free(name);
 				continue;
 			}
@@ -633,7 +635,9 @@ add_drawers_from_dir(char *dirname, char *name, int pos, PanelWidget *panel)
 						     newpanel);
 			} else {
 				char *p = strrchr(filename,'.');
-				if(p && strcmp(p,".desktop")==0)
+				if (p &&
+				    (strcmp(p,".desktop")==0 || 
+				     strcmp(p,".kdelnk")==0))
 					/*we load the applet at the right
 					  side, that is end of the drawer*/
 					load_launcher_applet(filename,
@@ -667,6 +671,10 @@ add_menu_to_panel (GtkWidget *widget, void *data)
 	if(g_file_exists("/etc/X11/wmconfig"))
 		flags |= MAIN_MENU_REDHAT|MAIN_MENU_REDHAT_SUB;
 
+	/*guess KDE menus*/
+	if(g_file_exists(KDE_MENUDIR))
+		flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
+
 	/*guess debian menus*/
 	if (g_file_exists("/etc/menu-methods/gnome"))
 		flags |= MAIN_MENU_DEBIAN|MAIN_MENU_DEBIAN_SUB;
@@ -749,6 +757,13 @@ edit_dentry(GtkWidget *widget, char *item_loc)
 	
 	g_return_if_fail(item_loc!=NULL);
 
+	dentry = gnome_desktop_entry_load(item_loc);
+	/* We'll screw up a KDE menu entry if we edit it */
+	if (dentry && dentry->is_kde) {
+		gnome_desktop_entry_free (dentry);
+		return;
+	}
+
 	dialog = gnome_property_box_new();
 	gtk_window_set_wmclass(GTK_WINDOW(dialog),
 			       "desktop_entry_properties","Panel");
@@ -759,7 +774,6 @@ edit_dentry(GtkWidget *widget, char *item_loc)
 	/*item loc will be alive all this time*/
 	gtk_object_set_data(o,"location",item_loc);
 
-	dentry = gnome_desktop_entry_load(item_loc);
 	if(dentry) {
 		gnome_dentry_edit_set_dentry(GNOME_DENTRY_EDIT(o),dentry);
 		gnome_desktop_entry_free(dentry);
@@ -783,6 +797,13 @@ edit_direntry(GtkWidget *widget, MenuFinfo *mf)
 	char *dirfile = g_concat_dir_and_file(mf->menudir, ".directory");
 	GnomeDesktopEntry *dentry;
 
+	dentry = gnome_desktop_entry_load_unconditional(dirfile);
+	/* We'll screw up a KDE menu entry if we edit it */
+	if (dentry && dentry->is_kde) {
+		gnome_desktop_entry_free (dentry);
+		return;
+	}
+
 	dialog = gnome_property_box_new();
 	gtk_window_set_wmclass(GTK_WINDOW(dialog),
 			       "desktop_entry_properties","Panel");
@@ -791,7 +812,6 @@ edit_direntry(GtkWidget *widget, MenuFinfo *mf)
 	
 	o = gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(dialog)->notebook));
 
-	dentry = gnome_desktop_entry_load_unconditional(dirfile);
 	if (dentry) {
 		gnome_dentry_edit_set_dentry(GNOME_DENTRY_EDIT(o), dentry);
 		gtk_object_set_data_full(o,"location",
@@ -913,7 +933,7 @@ show_item_menu(GtkWidget *item, GdkEventButton *bevent, ShowItemMenu *sim)
 					   sim->mf);
 		gtk_object_set_data(GTK_OBJECT(item),"prop_item",
 				    sim->prop_item);
-		setup_menuitem (sim->prop_item, 0, _("Properties ..."));
+		setup_menuitem (sim->prop_item, 0, _("Properties..."));
 		gtk_menu_append (GTK_MENU (sim->menu), sim->prop_item);
 	}
 	
@@ -1194,7 +1214,7 @@ setup_full_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title,
 		}
 
 		gtk_drag_source_set(menuitem,
-				    GDK_BUTTON1_MASK,
+				    GDK_BUTTON1_MASK|GDK_BUTTON2_MASK,
 				    menu_item_targets, 1,
 				    GDK_ACTION_COPY);
 
@@ -1221,7 +1241,7 @@ setup_directory_drag (GtkWidget *menuitem, char *directory)
 	};
 
 	gtk_drag_source_set(menuitem,
-			    GDK_BUTTON1_MASK,
+			    GDK_BUTTON1_MASK|GDK_BUTTON2_MASK,
 			    menu_item_targets, 1,
 			    GDK_ACTION_COPY);
 	
@@ -1244,7 +1264,7 @@ setup_internal_applet_drag (GtkWidget *menuitem, char *applet_type)
 		return;
 	
 	gtk_drag_source_set(menuitem,
-			    GDK_BUTTON1_MASK,
+			    GDK_BUTTON1_MASK|GDK_BUTTON2_MASK,
 			    menu_item_targets, 1,
 			    GDK_ACTION_COPY);
 	
@@ -1268,7 +1288,7 @@ setup_applet_drag (GtkWidget *menuitem, char *goad_id)
 		return;
 	
 	gtk_drag_source_set(menuitem,
-			    GDK_BUTTON1_MASK,
+			    GDK_BUTTON1_MASK|GDK_BUTTON2_MASK,
 			    menu_item_targets, 1,
 			    GDK_ACTION_COPY);
 	
@@ -2095,6 +2115,23 @@ create_debian_menu(GtkWidget *menu, int fake_submenus, int fake)
 	return menu;
 }
 
+static GtkWidget *
+create_kde_menu(GtkWidget *menu, int fake_submenus,
+		int force, int fake)
+{
+	if(!fake || menu) {
+		menu = create_menu_at (menu, 
+				       KDE_MENUDIR, FALSE,
+				       _("KDE menus"), 
+				       NULL,fake_submenus,
+				       force);
+	} else {
+		menu = create_fake_menu_at (KDE_MENUDIR, FALSE,
+					    _("KDE menus"),NULL);
+	}
+	return menu;
+}
+
 GtkWidget *
 create_panel_root_menu(GtkWidget *panel)
 {
@@ -2328,30 +2365,6 @@ make_panel_submenu (GtkWidget *menu, int fake_submenus)
 			   GTK_SIGNAL_FUNC(submenu_to_display),
 			   menuitem);
 
-	menuitem = gtk_menu_item_new ();
-	setup_menuitem (menuitem, 0, _("Add new panel"));
-	gtk_menu_append (GTK_MENU (menu), menuitem);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
-				   create_add_panel_submenu());
-
-	add_menu_separator(menu);
-
-	menuitem = gtk_menu_item_new ();
-	setup_menuitem (menuitem, 0, _("Convert to corner panel"));
-	gtk_menu_append (GTK_MENU (menu), menuitem);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-			   (GtkSignalFunc) convert_to_panel,
-			   GINT_TO_POINTER(CORNER_PANEL));
-	gtk_object_set_data(GTK_OBJECT(menu),"convcorn",menuitem);
-
-	menuitem = gtk_menu_item_new ();
-	setup_menuitem (menuitem, 0, _("Convert to edge panel"));
-	gtk_menu_append (GTK_MENU (menu), menuitem);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-			   (GtkSignalFunc) convert_to_panel,
-			   GINT_TO_POINTER(SNAPPED_PANEL));
-	gtk_object_set_data(GTK_OBJECT(menu),"convsnap",menuitem);
-
 	menuitem = gtk_menu_item_new ();
 	setup_menuitem (menuitem, 0, _("Add main menu"));
 	gtk_menu_append (GTK_MENU (menu), menuitem);
@@ -2414,6 +2427,29 @@ make_panel_submenu (GtkWidget *menu, int fake_submenus)
 			    GTK_SIGNAL_FUNC(panel_config_global), 
 			    NULL);
 
+	menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem, 0, _("Convert to corner panel"));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+			   (GtkSignalFunc) convert_to_panel,
+			   GINT_TO_POINTER(CORNER_PANEL));
+	gtk_object_set_data(GTK_OBJECT(menu),"convcorn",menuitem);
+
+	menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem, 0, _("Convert to edge panel"));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+			   (GtkSignalFunc) convert_to_panel,
+			   GINT_TO_POINTER(SNAPPED_PANEL));
+	gtk_object_set_data(GTK_OBJECT(menu),"convsnap",menuitem);
+
+	add_menu_separator(menu);
+
+        menuitem = gtk_menu_item_new ();
+	setup_menuitem (menuitem, 0, _("Create new panel"));
+	gtk_menu_append (GTK_MENU (menu), menuitem);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
+				   create_add_panel_submenu());
 }
 
 static void
@@ -2862,6 +2898,10 @@ create_root_menu(int fake_submenus, int flags)
 		root_menu = create_debian_menu(root_menu, fake_submenus, FALSE);
 		need_separ = TRUE;
 	}
+	if(flags&MAIN_MENU_KDE && !(flags&MAIN_MENU_KDE_SUB)) {
+		root_menu = create_kde_menu(root_menu,fake_submenus,FALSE, FALSE);
+		need_separ = TRUE;
+	}
 	/*others here*/
 	
 	if(!root_menu)
@@ -2931,6 +2971,30 @@ create_root_menu(int fake_submenus, int flags)
 					   menuitem);
 		}
 	}
+	if(flags&MAIN_MENU_KDE && flags&MAIN_MENU_KDE_SUB) {
+		GtkWidget *pixmap = NULL;
+		if(need_separ)
+			add_menu_separator(root_menu);
+		need_separ = FALSE;
+		menu = create_kde_menu(NULL, fake_submenus, TRUE, TRUE);
+		if (g_file_exists("/usr/share/icons/exec.xpm")) {
+			pixmap = gnome_stock_pixmap_widget_at_size (NULL, "/usr/share/icons/exec.xpm",
+								    SMALL_ICON_SIZE,
+								    SMALL_ICON_SIZE);
+			if (pixmap)
+				gtk_widget_show (pixmap);
+		}
+		menuitem = gtk_menu_item_new ();
+		setup_menuitem (menuitem, pixmap, _("KDE menus"));
+		gtk_menu_append (GTK_MENU (root_menu), menuitem);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
+		gtk_signal_connect(GTK_OBJECT(menu),"show",
+				   GTK_SIGNAL_FUNC(rh_submenu_to_display),
+				   menuitem);
+		gtk_signal_connect(GTK_OBJECT(menu),"show",
+				   GTK_SIGNAL_FUNC(submenu_to_display),
+				   menuitem);
+	}
 	add_special_entries (root_menu, fake_submenus);
 	
 	return root_menu;
@@ -3128,6 +3192,8 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 	GtkWidget *user_sub = gtk_object_get_data(GTK_OBJECT(widget), "user_sub");
 	GtkWidget *redhat_off = gtk_object_get_data(GTK_OBJECT(widget), "redhat_off");
 	GtkWidget *redhat_sub = gtk_object_get_data(GTK_OBJECT(widget), "redhat_sub");
+ 	GtkWidget *kde_off = gtk_object_get_data(GTK_OBJECT(widget), "kde_off");
+ 	GtkWidget *kde_sub = gtk_object_get_data(GTK_OBJECT(widget), "kde_sub");
 	GtkWidget *debian_off = gtk_object_get_data(GTK_OBJECT(widget), "debian_off");
 	GtkWidget *debian_sub = gtk_object_get_data(GTK_OBJECT(widget), "debian_sub");
 	GtkWidget *pathentry = gtk_object_get_data(GTK_OBJECT(widget), "path");
@@ -3183,6 +3249,14 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 		menu->main_menu_flags |= MAIN_MENU_DEBIAN;
 		menu->main_menu_flags &=~ MAIN_MENU_DEBIAN_SUB;
 	}
+	if(GTK_TOGGLE_BUTTON(kde_off)->active)
+		menu->main_menu_flags &=~ (MAIN_MENU_KDE|MAIN_MENU_KDE_SUB);
+	else if(GTK_TOGGLE_BUTTON(kde_sub)->active)
+		menu->main_menu_flags |= MAIN_MENU_KDE|MAIN_MENU_KDE_SUB;
+	else {
+		menu->main_menu_flags |= MAIN_MENU_KDE;
+		menu->main_menu_flags &=~ MAIN_MENU_KDE_SUB;
+	}
 
 	if(menu->menu)
 		gtk_widget_destroy(menu->menu);	
@@ -3358,7 +3432,7 @@ create_properties_dialog(Menu *menu)
 	gtk_object_set_data(GTK_OBJECT(dialog),"main_frame",f);
 	gtk_box_pack_start(GTK_BOX(vbox),f,FALSE,FALSE,0);
 	
-	table = gtk_table_new(3,4,FALSE);
+	table = gtk_table_new(5,4,FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(table),GNOME_PAD_SMALL);
 	gtk_container_add(GTK_CONTAINER(f),table);
 
@@ -3374,7 +3448,11 @@ create_properties_dialog(Menu *menu)
 			      _("AnotherLevel menu (if found): "),"redhat",
 			      menu->main_menu_flags&MAIN_MENU_REDHAT,
 			      menu->main_menu_flags&MAIN_MENU_REDHAT_SUB);
-	add_menu_type_options(GTK_OBJECT(dialog),GTK_TABLE(table),3,
+ 	add_menu_type_options(GTK_OBJECT(dialog),GTK_TABLE(table),3,
+ 			      _("KDE menu (if found): "),"kde",
+ 			      menu->main_menu_flags&MAIN_MENU_KDE,
+ 			      menu->main_menu_flags&MAIN_MENU_KDE_SUB);
+	add_menu_type_options(GTK_OBJECT(dialog),GTK_TABLE(table),4,
 			      _("Debian menu (if found): "),"debian",
 			      menu->main_menu_flags&MAIN_MENU_DEBIAN,
 			      menu->main_menu_flags&MAIN_MENU_DEBIAN_SUB);
